package org.jenkinsci.plugins.p4_client;

import hudson.FilePath.FileCallable;
import hudson.model.TaskListener;
import hudson.remoting.VirtualChannel;

import java.io.File;
import java.io.IOException;
import java.util.Map;
import java.util.logging.Logger;

import org.jenkinsci.plugins.p4_client.client.ConnectionHelper;
import org.jenkinsci.plugins.p4_client.workspace.Workspace;

public class CheckoutTask implements FileCallable<Boolean> {

	private static final long serialVersionUID = 1L;
	private static Logger logger = Logger.getLogger(CheckoutTask.class
			.getName());

	private ConnectionHelper p4;

	private int change;
	private int review;
	private CheckoutStatus status = CheckoutStatus.HEAD;

	/**
	 * Constructor
	 * 
	 * @param config
	 *            - Server connection details
	 * @param auth
	 *            - Server login details
	 */
	public CheckoutTask(String credentialID, Workspace config,
			TaskListener listener) {
		try {
			p4 = new ConnectionHelper(credentialID, listener);
			p4.login();
			p4.setClient(config);
		} catch (Exception e) {
			logger.severe("Unable to connect.");
			e.printStackTrace();
		}
	}

	/**
	 * Invoke sync on build node (master or remote node).
	 * 
	 * @return true if updated, false if no change.
	 */
	public Boolean invoke(File workspace, VirtualChannel channel)
			throws IOException {

		logger.info("P4:CheckoutTask sync...");
		boolean success = true;

		try {
			success &= p4.tidyWorkspace();

			switch (status) {
			case HEAD:
				success &= p4.syncFiles();
				break;

			case COMMITTED:			
			case SUBMITTED:
				success &= p4.syncFiles(change);
				break;

			case SHELVED:
				success &= p4.unshelveFiles(review);
				break;

			default:
				logger.severe("P4:CheckoutTask unknown status: " + status);
				return false;
			}

			p4.disconnect();
		} catch (Exception e) {
			logger.severe("Unable to sync workspace.");
			e.printStackTrace();
			return false;
		}

		return success;
	}

	/**
	 * Takes the Build Variables set for the Jenkins Action and sets status,
	 * change and review numbers.
	 * 
	 * @param buildVariables
	 */
	public void setBuildVariables(Map<String, String> map) {
		if (map == null) {
			return;
		}

		if (map.containsKey("status")) {
			String value = map.get("status");
			status = CheckoutStatus.parse(value);
		}

		if (map.containsKey("change")) {
			String value = map.get("change");
			try {
				change = Integer.parseInt(value);
			} catch (NumberFormatException e) {
				change = 0;
			}
		}

		if (map.containsKey("review")) {
			String value = map.get("review");
			try {
				review = Integer.parseInt(value);
			} catch (NumberFormatException e) {
				review = 0;
			}
		}
	}

}
